# ! -*- coding: utf-8 -*-

import base64
import datetime
from django.conf import settings
from django.contrib.auth import login
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from jmb.core.middleware.force_response import ForceResponse
from jmb.core.utils.cmsutils import get_abs_url_from_slug
from jmb.core.utils.functions import import_class

from acomea.contact.utils.update import set_contact_in_registration
from acomea.premium.cms.models import PremiumBlockSettings, PremiumRegistrationSettings, PremiumSetUserInGroupSettings
from acomea.profilo.utils import (
    send_email_registrazione, send_email_attivazione, check_activation_url,
    set_user_as_active, iscrizione_a_newsletter, aggiunta_a_gruppo_ar, aggiunta_a_gruppo_lead,
    set_bonifica_in_contact_user
)
from acomea.premium.cms.forms import PremiumRegistrationPluginForm
from acomea.contact.models import ContactGroup


class PremiumBlockPlugin(CMSPluginBase):
    module = _('Premium')
    model = PremiumBlockSettings
    name = _('Premium Block')
    admin_preview = False
    render_template = "bootstrap3/premium/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        redirect_to = get_abs_url_from_slug(instance.redirect_to)
        request.session["premium_from_absolute_uri"] = request.build_absolute_uri()
        # Checks
        if user.is_anonymous():
            raise ForceResponse(HttpResponseRedirect(redirect_to))
        if not user.contact.auth_materiale_commerciale and not user.contact.auth_premium:
            user.contact.create_premium_url(request.session["premium_from_absolute_uri"])
            raise ForceResponse(HttpResponseRedirect(redirect_to))
        elif not user.contact.auth_materiale_commerciale and user.contact.auth_premium:
            user.contact.create_premium_url(request.session["premium_from_absolute_uri"])
            raise ForceResponse(HttpResponseRedirect(redirect_to))
        elif user.contact.auth_materiale_commerciale and not user.contact.auth_premium:
            user.contact.create_premium_url(request.session["premium_from_absolute_uri"])
            user.contact.auth_premium = True
            user.contact.save()
        elif user.contact.auth_materiale_commerciale and user.contact.auth_premium:
            user.contact.create_premium_url(request.session["premium_from_absolute_uri"])
        # Aggiunta gruppo se necessario
        if instance.group and user.contact:
            if instance.group not in user.contact.groups.all():
                user.contact.groups.add(instance.group)
        # Context update
        context.update({
            'object': instance,
            'placeholder': placeholder,
        })
        return context


class PremiumRegistrationPlugin(CMSPluginBase):
    module = _('Premium')
    model = PremiumRegistrationSettings
    name = _('Premium Registration Form')
    admin_preview = False
    render_template = "bootstrap3/premium/cms_plugins/empty.html"
    form = PremiumRegistrationPluginForm
    fieldsets = (
        (_('general info'), {'fields': (
            ('slug', 'status'), 'sent_from', 'missing_session_redirect_to', 'has_social', 'contact_group'
        )}),
        (_('registration'), {'fields': (
            'registration_body', 'registration_disclaimer', 'registration_done',
            'activation_done_template', 'post_registration_email', 'email_activation_done_template',
        )}),
        (_('disclaimer'), {'fields': (
            'disclaimer_title', 'disclaimer_body',
        )}),
    )

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        user_type = "anonymous" if user.is_anonymous() else "authenticated"
        self.render_template = settings.PREMIUM_CMSPLUGIN_PREMIUMREGISTRATION_FORM_TEMPLATES[user_type]
        form_class = import_class(settings.PREMIUM_CMSPLUGIN_PREMIUMREGISTRATION_FORM_CLASSES[user_type])
        plugin_form = form_class()

        premium_url = request.session.get(
            "premium_from_absolute_uri",
            get_abs_url_from_slug(instance.missing_session_redirect_to)
        )

        # Invio form registrazione per utenti anonimi
        if request.method == 'POST' and user_type == 'anonymous' and 'registration_premium_new_submit' in request.POST:
            plugin_form = form_class(data=request.POST, files=request.FILES)
            if plugin_form.is_valid():
                new_user = plugin_form.save()
                set_contact_in_registration(new_user, plugin_form.cleaned_data)
                if instance.contact_group:
                    new_user.contact.groups.add(instance.contact_group)
                    new_user.contact = new_user.contact.get_reload_from_db()  # to consider signal "contact_group_signal" manipulation
                send_email_registrazione(
                    new_user.profilo, instance, request,
                    premium_url=premium_url
                )
                utm_campaign = request.COOKIES.get('utm_campaign', None)
                utm_source = request.COOKIES.get('utm_source', None)
                new_user.contact.assign_campaign_source(utm_campaign, utm_source)

                iscrizione_a_newsletter(plugin_form.cleaned_data, new_user)
                aggiunta_a_gruppo_ar(user=new_user)
                aggiunta_a_gruppo_lead(user=new_user)

        # Invio form disclaimer per utenti autenticati
        elif request.method == 'POST' and user_type == 'authenticated' and 'disclaimer_premium_submit' in request.POST:
            plugin_form = form_class(data=request.POST, files=request.FILES)
            if plugin_form.is_valid():
                user.contact.auth_materiale_commerciale = True
                user.contact.auth_premium = True
                user.contact.save()

                raise ForceResponse(HttpResponseRedirect(premium_url))

        # Rifiuto form disclaimer per utenti autenticati
        elif request.method == 'POST' and user_type == 'authenticated' and 'disclaimer_premium_discard' in request.POST:
            raise ForceResponse(HttpResponseRedirect(premium_url))

        elif request.method == 'GET':
            path_premium_url = request.GET.get('premium_url', None)
            # autenticazione: pagina di atterraggio dopo click su link
            # contenuto in mail di conferma registrazione e login automatico utente
            if path_premium_url:
                _user = check_activation_url(request)
                if _user:
                    set_user_as_active(_user)
                    set_bonifica_in_contact_user(_user)
                    send_email_attivazione(_user.profilo, instance, request)
                    _user.contact.auth_materiale_commerciale = True
                    _user.contact.auth_trattamento_dati = True
                    _user.contact.auth_premium = True
                    _user.contact.save()
                    # Con SSO non si può autenticare in modo automatico, non abbiamo la pwd dopo il link da email
                    # _user.backend = 'acomea.sso.auth.SSOBackend'
                    # login(request, _user)
                    premium_url = base64.b64decode(path_premium_url).rstrip('/')
                    premium_url += "&firstAccess=true" if len(premium_url.split("?")) > 1 else "/?firstAccess=true"

                    response = HttpResponseRedirect(premium_url)
                    kw = {'domain': '.acomea.it'}
                    response.delete_cookie('utm_campaign', **kw)
                    response.delete_cookie('utm_source', **kw)

                    raise ForceResponse(response)
            elif user.is_authenticated() and user.contact.auth_materiale_commerciale and user.contact.auth_premium:

                raise ForceResponse(HttpResponseRedirect(premium_url))

        context.update({
            'object': instance,
            'plugin_form': plugin_form,
            'placeholder': placeholder,
            'site_domain': settings.SITE_DOMAIN,
            'social_url': reverse(
                'tracerequest:trace_request_post_social_login'
            ) + '?redirect={}&premium=true'.format(premium_url)
        })
        return context


class PremiumSetUserInGroupPlugin(CMSPluginBase):
    module = _('Premium')
    model = PremiumSetUserInGroupSettings
    name = _('Premium set contact in group')
    admin_preview = False
    render_template = "bootstrap3/premium/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        if user.is_authenticated() and hasattr(user, 'contact'):
            if instance.group not in user.contact.groups.all():
                user.contact.groups.add(instance.group)  # signal "contact_group_signal" change "first_group" field, to use contact instace remember to reload
        context.update({'object': instance, 'placeholder': placeholder})
        return context


plugin_pool.register_plugin(PremiumBlockPlugin)
plugin_pool.register_plugin(PremiumRegistrationPlugin)
plugin_pool.register_plugin(PremiumSetUserInGroupPlugin)
